/* artywz.v
   2019.5.10   Ver0.0
   Naoki Hirayama
*/
`timescale 1ns/1ps
//`define LED_DISPLAY

module artywz(
    input CLK,
    input xRESET_N,
    input SW,
    input [3:0] BTN,
    output [7:0] LED,
    output [18:0] IOZ);

    wire xRESET_P = ~xRESET_N;


    wire xWSTROBE_P;
    wire xRSTROBE_P;
    wire [7:0] xPORTID_P;
    wire [7:0] xOUTPORT_P;

    genvar i;

    wzeta WZeta(
        .CLK(CLK),
        .xINPORT_P(IOZ[8:1]),
        .xRESET_P(xRESET_P),
`ifdef PARITY_CHECK
        .xSW0_P(SW),
        .xPCK_P(IOZ[0]),
`endif
        .xWSTROBE_P(xWSTROBE_P),
	.xRSTROBE_P(xRSTROBE_P),
        .xPORTID_P(xPORTID_P),
        .xOUTPORT_P(xOUTPORT_P));

`ifdef LED_DISPLAY

    led_arty LED_arty(
        .CLK(CLK),
        .RESET(xRESET_P),
        .STROBE(xWSTROBE_P),
        .PORTID(xPORTID_P),
        .DATA(xOUTPORT_P),
        .BTN(BTN),
        .LED(LED));

    generate for(i=9 ; i<=18 ; i=i+1 ) begin
        assign IOZ[i] = xRSTROBE_P;
        end
    endgenerate

`else
    assign IOZ[18] = xWSTROBE_P;
    assign IOZ[17:10] = xPORTID_P;
    assign LED = xOUTPORT_P;
`endif

`ifndef PARITY_CHECK
    assign IOZ[0] = SW;
`endif

endmodule

module led_arty(CLK,RESET,STROBE,PORTID,DATA,BTN,LED);
    input CLK;
    input RESET;
    input STROBE;
    input [7:0] PORTID;
    input [7:0] DATA;
    input [3:0] BTN;
    output [7:0] LED;

    reg [7:0] st,r0,r1,r2,r3;

    always @(posedge CLK) begin
        st <= RESET ? 0 : (STROBE && PORTID==8'h00) ? DATA : st;
        r0 <= RESET ? 0 : (STROBE && PORTID==8'h01) ? DATA : r0;
        r1 <= RESET ? 0 : (STROBE && PORTID==8'h02) ? DATA : r1;
        r2 <= RESET ? 0 : (STROBE && PORTID==8'h03) ? DATA : r2;
        r3 <= RESET ? 0 : (STROBE && PORTID==8'h04) ? DATA : r3;
    end

    assign LED = BTN[0] ? r0 :
                 BTN[1] ? r1 :
                 BTN[2] ? r2 :
                 BTN[3] ? r3 : st;

endmodule // led_arty


    

