/* wzpmem.v
   2019.4.27   Ver0.0
   Naoki Hirayama
*/
`timescale 1ns/1ps
`ifndef PC_WIDTH
    `define PC_WIDTH 11
`endif

module rom_style (clk, en, addrA, dout);
    parameter ADDR_WIDTH = 8;
    parameter DATA_WIDTH = 16;

    input       clk;
    input       en;
    input [ADDR_WIDTH-1:0] addrA;
    output reg [DATA_WIDTH-1:0] dout;

    (* rom_style = "block" *)
    reg [DATA_WIDTH-1:0] rom [2**ADDR_WIDTH-1:0];

    initial $readmemh("pmem.mem", rom);

    always @(posedge clk) begin
        if (en)
          dout <= rom[addrA];
    end
endmodule // rom_style

module wzpmem(CLK, xPC_P, xIR_P);
    parameter M_WIDTH = 8;
    input CLK;
    input [`PC_WIDTH-1:0] xPC_P;
    output [M_WIDTH-1:0] xIR_P;

    wire [M_WIDTH-1:0] dout;

    rom_style #(
        .ADDR_WIDTH(`PC_WIDTH),
        .DATA_WIDTH(M_WIDTH)
    ) program_rom (
        .clk(CLK),
        .en(1'b1),
        .addrA(xPC_P),
        .dout(dout)
    );

    assign xIR_P = dout;

endmodule
