#!/usr/bin/perl
##########################################################
# WZeta Project https://wzeta.idletime.tokyo/
# Naoki Hirayama 2019/5/9
##########################################################

##### Optiopn #####
$fetch = "_fetch";
$store = "_store";
$stack = "_stack";
##########################################################

$pass = 0;
$fail = 0;

foreach( @ARGV ) {
    if(/^fetch$/) { $fetch = "_fetch"; }
    if(/^store$/) { $store = "_store"; }
    if(/^stack$/) { $store = "_stack"; }
}

$MAKE = "make icarus$fetch$store$stack >& /dev/null";
$EXE = "./simwz |";

foreach( @ARGV ) {
    if(/^xsim$/) {
        $MAKE = "make xsim$fetch$store$stack >& /dev/null";
        $EXE = "xsim -R simsim |";
    }
}

sub dotest {
    if($tname eq '' || $check eq '') { die; }

    system("./wzsim a $tname.awz");
    system("rm -rf pmem.mem");
    system("mv $tname.mem pmem.mem");
    system($MAKE);
    open(IN,$EXE);

    $flag = 0;
    while(<IN>) {
        if( /$check/ ) {$flag = 1;}
    }
    close(IN);
    if( $flag==1 ) {
        print "$tname ok!\n";
        $pass++;
    } else {
        print "$tname *** error ***\n";
        $fail++;
    }
}

##########################################################
@list = qw/R0=12 R0=06 R0=02 R0=02 R0=02 R0=02/;
for( $i = 1 ; $i<=6 ; $i++ ) {
    $tname = "opr$i";
    $check = shift( @list );
    &dotest;
}

##########################################################
@list = qw/R0=44 R0=44 R0=44 R0=0a/;
for( $i = 1 ; $i<=4 ; $i++ ) {
    $tname = "jump$i";
    $check = shift( @list );
    &dotest;
}

##########################################################
@list = qw/R0=44 R0=44 R0=55 R0=11/;
for( $i = 1 ; $i<=4 ; $i++ ) {
    $tname = "call$i";
    $check = shift( @list );
    &dotest;
}

##########################################################
@list = qw/R0=37 R0=79 R0=78 R0=ff/;
for( $i = 1 ; $i<=4 ; $i++ ) {
    $tname = "add$i";
    $check = shift( @list );
    &dotest;
}

##########################################################
@list = qw/R0=11 R0=12 R0=01/;
for( $i = 1 ; $i<=3 ; $i++ ) {
    $tname = "sub$i";
    $check = shift( @list );
    &dotest;
}

##########################################################
$tname = "andorxor";
$check = "R0=01";
&dotest;

##########################################################
$tname = "compare1";
$check = "R0=aa";
&dotest;

##########################################################
$tname = "compare2";
$check = "R0=aa";
&dotest;

##########################################################
$tname = "loadret";
$check = "R0=0f";
&dotest;

##########################################################
$tname = "address";
$check = "R0=fe";
&dotest;

##########################################################
@list = qw/R0=12 R0=62 R0=04 R0=63 R0=77/;
for( $i = 1 ; $i<=5 ; $i++ ) {
    $tname = "input$i";
    $check = shift( @list );
    &dotest;
}

if($fetch eq "_fetch"  &&  $store eq "_store") {

##########################################################
$tname = "fetchstore";
$check = "R0=de";
&dotest;

}

if($fetch eq "_fetch") {
##########################################################
$tname = "aes1";
$check = "R0=8e";
&dotest;

##########################################################
$tname = "aes2";
$check = "R0=00";
&dotest;

##########################################################
$tname = "aes3";
$check = "R0=8e";
&dotest;

##########################################################
$tname = "encdec1";
$check = "R0=00 R1=11 R2=22 R3=33 R4=44 R5=55 R6=66 R7=77 R8=88 R9=99 R10=aa R11=bb R12=cc R13=dd R14=ee R15=ff";
&dotest;

##########################################################
$tname = "encdec2";
$check = "R0=00 R1=11 R2=22 R3=33 R4=44 R5=55 R6=66 R7=77 R8=88 R9=99 R10=aa R11=bb R12=cc R13=dd R14=ee R15=ff";
&dotest;

##########################################################
$tname = "encdec3";
$check = "R0=00 R1=11 R2=22 R3=33 R4=44 R5=55 R6=66 R7=77 R8=88 R9=99 R10=aa R11=bb R12=cc R13=dd R14=ee R15=ff";
&dotest;

}

if($stack eq "_stack") {
##########################################################
$tname = "stack1";
$check = "R0=03";
&dotest;

##########################################################
$tname = "stack2";
$check = "R0=88";
&dotest;
}

$total = $pass + $fail;
print "pass=$pass , fail=$fail , total=$total\n";
