/* wzab.v
   2019.6.03   Ver0.0
   Naoki Hirayama
*/
`timescale 1ns/1ps

`ifndef SPM_WIDTH
    `define SPM_WIDTH 7
`endif

module wzab(
    input CLK,
    input [7:0] xIRD_P,      // CTL
    input [7:0] xREGDO_P,    // REG
    input [1:0] xSTAGE_P,    // CTL
    input xCF_P,             // CTL
    input [7:0] xINPORT_P,   // GAIBU
    output [7:0] xOUTPORT_P, // GAIBU
`ifdef OP_STORE
    output [15:0] xABREG_P,  // CTL
`else
   output [`SPM_WIDTH-1:0] xABREG_P,  // CTL
`endif	    
    output [7:0] xPORTID_P,  // GAIBU
    output xCY_P,            // CTL
    output xZF_P,            // CTL
    output [7:0] xREGDI_P);  // REG

    reg [7:0] A, B;

    always @(posedge CLK) begin
        A <= xSTAGE_P[0] ? (xIRD_P[7] ? xREGDO_P : {1'b0 , xIRD_P[6:0]}) : A;
        B <= xSTAGE_P[1] ? (xIRD_P[7] ? xREGDO_P : {1'b0 , xIRD_P[6:0]}) : B;
    end

    wire [7:0] ADD, ADDR, LG;
    assign ADDR = xIRD_P[1] ? ~B : B;
    assign { xCY_P , ADD } = A + ADDR + (xIRD_P[0] | (xIRD_P[2]&xCF_P));

    wire [7:0] LG_A;
    assign LG_A = xIRD_P[7] ? xINPORT_P : A;

`ifdef OP_STORE
    assign xABREG_P  = {B[7:0] , A[7:0]};
    assign LG = (xIRD_P[1:0]==2'b00) ? LG_A :
                (xIRD_P[1:0]==2'b01) ? (LG_A & B) :
                (xIRD_P[1:0]==2'b10) ? (LG_A ^ B) : (LG_A | B);
`else
    generate
    if(`SPM_WIDTH > 8) assign xABREG_P  = {B[`SPM_WIDTH-9:0] , A[7:0]};
    else assign xABREG_P  = A[`SPM_WIDTH-1:0];
    endgenerate

    assign LG = (xIRD_P[1:0]==2'b00) ? LG_A :
                (xIRD_P[1:0]==2'b01) ? (LG_A & B) :
                (xIRD_P[1:0]==2'b10) ? (LG_A ^ B) : (LG_A | B);
`endif

    assign xREGDI_P = xIRD_P[6] ? LG : ADD;
    assign xZF_P = ~|xREGDI_P;

    assign xPORTID_P  = A;
    assign xOUTPORT_P = B;

endmodule // wzab
