/* wzctl.v
   2019.5.10   Ver0.0
   2019.5.18   Ver0.1
   Naoki Hirayama
*/
`timescale 1ns/1ps
//`define USE_XILINX_RAM_STYLE
//`define MULT_STACK 32

`ifdef PARITY_CHECK
    `define MBUS_WIDTH 9
`else
    `define MBUS_WIDTH 8
`endif

`ifndef SPM_WIDTH
    `define SPM_WIDTH 7
`endif

`ifndef PC_WIDTH
    `define PC_WIDTH 11
`endif

module wzctl(
    input CLK,
    input xRESET_P,                  // GAIBU
    input xCY_P,                     // AB
    input xZF_P,                     // AB
`ifdef OP_STORE
    input [15:0] xABREG_P,           // AB
`else
    input [`SPM_WIDTH-1:0] xABREG_P, // AB
`endif	     
    input [`MBUS_WIDTH-1:0] xIR_P,   // PMEM
`ifdef PARITY_CHECK	     
    input xSW0_P,                    // GAIBU
    output xPCK_P,                   // GAIBU
`endif	     
    output xCF_P,                    // AB
    output [`PC_WIDTH-1:0] xPC_P,    // PMEM
    output [`SPM_WIDTH-1:0] xREGRA_P, // REG
    output [`SPM_WIDTH-1:0] xREGWA_P, // REG
    output [7:0] xIRD_P,             // AB
    output xRSTROBE_P,              // GAIBU
    output xWSTROBE_P,              // GAIBU
`ifdef OP_OUTPUT2	     
    output xWSTROBE2_P,             // GAIBU
`endif	     
    output xREGWE_P,                // REG
    output [1:0] xSTAGE_P);         // AB

    reg [`PC_WIDTH-1:0] PC; // program counter
    reg [3:0] reset = 4'b1111;
    reg CF,ZF;
    reg [`MBUS_WIDTH-1:0] IR;
    reg [7:0] IRD;
    reg [1:0] S;
  
`ifdef PARITY_CHECK
    reg P;
    always @( posedge CLK) P <= xRESET_P ? 0 : (^IR | P) & ~xSW0_P;
    assign xPCK_P = P;
`endif        

    wire stage0, stage1, stage2;
    assign stage0 = (S==2'b00) ? 1 : 0;
    assign stage1 = (S==2'b01) ? 1 : 0;
    assign stage2 = (S==2'b10) ? 1 : 0;

    wire cond;
    wire uncond, cfcond, zfcond;

    assign uncond = ~(IR[1]|IR[2]);
    assign cfcond = ~(CF^IR[0])&IR[1];
    assign zfcond = ~(ZF^IR[0])&IR[2];
    assign cond  = (uncond | cfcond | zfcond) & stage1;

    wire call,jump,ret;
    assign call = cond & IR[3];
    assign jump = cond & IR[4];
    assign ret  = cond & IR[5];

`ifndef MULTI_STACK
    reg [`PC_WIDTH-3:0] R;  // return address
    always @( posedge CLK) R <= call ? PC[`PC_WIDTH-1:2] : R;
`else
    wire [`PC_WIDTH-3:0] R;  // return address
        smem #(.pc_width(`PC_WIDTH-2), .stack_width(`MULTI_STACK))
        Smem (.CLK(CLK), .RESET(xRESET_P), .CALL(call), .RETURN(ret), .DI(PC[`PC_WIDTH-1:2]), .DO(R));
`endif
    generate
    if( `PC_WIDTH > 9 ) assign xPC_P = ret ? { R , 2'b00 } : jump ? {IRD[`PC_WIDTH-10:0] , xABREG_P[6:0] , 2'b00} : PC;
    else assign xPC_P = ret ? { R , 2'b00 } : jump ? {xABREG_P[`PC_WIDTH-3:0] , 2'b00} : PC;
    endgenerate

`ifdef OP_FETCH
    reg FB; // Fetch Bit
    assign xREGRA_P = FB ? xABREG_P[`SPM_WIDTH-1:0] : IR[6:0];
    always @( posedge CLK) begin
        FB <= stage2&IRD[6]&IRD[2]&~IRD[1]&IRD[0] ? 1 : 0;
    end
`else
    assign xREGRA_P = IR[6:0];
`endif


`ifdef OP_STORE
    wire idwrite;
    assign idwrite = stage2&IRD[6]&IRD[2]&~IRD[1]&~IRD[0] ? 1 : 0;
    generate
        if(`SPM_WIDTH > 8) assign xREGWA_P = idwrite ? {IR[`SPM_WIDTH-9:0] , xABREG_P[15:8] } : IR[6:0];
        else assign xREGWA_P = idwrite ? xABREG_P[`SPM_WIDTH-1:0] : IR[6:0];
    endgenerate
`else
    assign xREGWA_P = IR[6:0];
`endif

    assign xIRD_P = IRD;

    always @( posedge CLK) begin
        reset[0] <= xRESET_P;
        reset[1] <= reset[0];
        reset[2] <= reset[1];
        reset[3] <= reset[2];
        PC <= reset[0] ? 0 : xPC_P+1;
        CF <= stage2 ? xCY_P : CF;
        ZF <= stage2 ? xZF_P : ZF;
        IR <= reset[1] ? 0 : xIR_P;
        IRD <= IR;
        S <= reset[3] ? 0 : S+1;
    end

    assign xCF_P = CF;

    assign xRSTROBE_P = stage2 & IRD[7];
    assign xWSTROBE_P = stage2 & IRD[2] & IRD[1] & IRD[0];
`ifdef OP_OUTPUT2	     
    assign xWSTROBE2_P = stage2 & IRD[6] & IRD[2] & IRD[1] & ~IRD[0];
`endif	     
    assign xREGWE_P = stage2 & IR[7];

    assign xSTAGE_P[0] = stage0;
    assign xSTAGE_P[1] = stage1;

endmodule // wzctl

module smem(CLK, RESET, CALL, RETURN, DI, DO);
    parameter pc_width = 7;
    parameter stack_width = 1;

    input CLK;
    input RESET;
    input CALL;
    input RETURN;
    input [pc_width-1:0] DI;
    output [pc_width-1:0] DO;

    reg [pc_width-1:0] dout;
    reg [stack_width-1:0] sp;

`ifdef USE_XILINX_RAM_STYLE
    (* ram_style = "distributed" *)
`endif
    reg [pc_width-1:0] ram [2**stack_width-1:0];
    wire [stack_width-1:0]  sp_plus_one;
    assign sp_plus_one = sp + 1;

    always @(posedge CLK) begin
        sp <= RESET ? 0 : CALL ? sp_plus_one : RETURN ? sp-1 : sp;        
        if(CALL) ram [sp_plus_one] = DI;
        else dout = ram [sp];
    end

    assign DO = dout;

endmodule
