/* wzeta.v
   2019.5.25   Ver0.0
   Naoki Hirayama
*/
`timescale 1ns/1ps

`ifdef PARITY_CHECK
    `define MBUS_WIDTH 9
`else
    `define MBUS_WIDTH 8
`endif

`ifndef SPM_WIDTH
    `define SPM_WIDTH 7
`endif

`ifndef PC_WIDTH
    `define PC_WIDTH 11
`endif

module wzeta(
    input CLK,
    input [7:0] xINPORT_P,
    input xRESET_P,
`ifdef PARITY_CHECK
    input xSW0_P,
`endif
    output [7:0] xPORTID_P,
    output [7:0] xOUTPORT_P,
`ifdef PARITY_CHECK
    output xPCK_P,
`endif
    output xWSTROBE_P,
`ifdef OP_OUTPUT2
    output xWSTROBE2_P,
`endif
    output xRSTROBE_P);

    parameter PROGFILE = "pmem.mem";

    wire [`PC_WIDTH-1:0] xPC_P;
    wire [`MBUS_WIDTH-1:0] xIR_P;
    wire [`SPM_WIDTH-1:0] xREGRA_P, xREGWA_P;
    wire xREGWE_P;
    wire [7:0] xREGDI_P, xREGDO_P;
    wire [7:0] xIRD_P;
    wire [1:0] xSTAGE_P;
    wire xCF_P,xCY_P,xZF_P;
`ifdef OP_STORE
    wire [15:0] xABREG_P;
`else
    wire [`SPM_WIDTH-1:0] xABREG_P;
`endif

    wzctl  WZctl(
        .CLK(CLK),
        .xRESET_P(xRESET_P),
        .xCY_P(xCY_P),
        .xZF_P(xZF_P),
        .xABREG_P(xABREG_P),
        .xIR_P(xIR_P),
`ifdef PARITY_CHECK
        .xSW0_P(xSW0_P),
`endif
        .xCF_P(xCF_P),
        .xPC_P(xPC_P),
        .xREGRA_P(xREGRA_P),
        .xREGWA_P(xREGWA_P),
        .xIRD_P(xIRD_P),
`ifdef PARITY_CHECK
        .xPCK_P(xPCK_P),
`endif
        .xRSTROBE_P(xRSTROBE_P),
        .xWSTROBE_P(xWSTROBE_P),
`ifdef OP_OUTPUT2
        .xWSTROBE2_P(xWSTROBE2_P),
`endif
        .xREGWE_P(xREGWE_P),
        .xSTAGE_P(xSTAGE_P));

    wzab WZab(
        .CLK(CLK),
        .xIRD_P(xIRD_P),
        .xREGDO_P(xREGDO_P),
        .xSTAGE_P(xSTAGE_P),
        .xCF_P(xCF_P),
        .xINPORT_P(xINPORT_P),
        .xOUTPORT_P(xOUTPORT_P),
        .xABREG_P(xABREG_P),
        .xPORTID_P(xPORTID_P),
        .xCY_P(xCY_P),
        .xZF_P(xZF_P),
        .xREGDI_P(xREGDI_P));

    wzpmem #(.M_WIDTH(`MBUS_WIDTH),.PROGFILE(PROGFILE))
        WZpmem(
        .CLK(CLK),
        .xPC_P(xPC_P),
        .xIR_P(xIR_P));

    wzreg WZreg(
        .CLK(CLK),
        .xREGRA_P(xREGRA_P),
        .xREGWA_P(xREGWA_P),
        .xREGWE_P(xREGWE_P),
        .xREGDI_P(xREGDI_P),
        .xREGDO_P(xREGDO_P));

endmodule // wzeta

