/* WZeta assembler & simulator
   WZeta Project https://wzeta.idletime.tokyo/
   Naoki Hirayama
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "wzsim.h"

extern unsigned int *PCODE;
extern unsigned long long *PDEBUG;
extern char** LDEBUG;
extern int adr_line;

unsigned long long time;
int CF,ZF,PC0;
unsigned char *R;
int STACK[MAX_SP];
int SP;

extern unsigned char Sbox[256];
extern unsigned char iSbox[256];
extern unsigned char exkey[240];

extern void exit_error();

void print_register(unsigned long long debug) {
    if(fp==NULL) return;
    if(debug & 0x00000001) fprintf(fp,"R0=%02X ",R[0]);
    if(debug & 0x00000002) fprintf(fp,"R1=%02X ",R[1]);
    if(debug & 0x00000004) fprintf(fp,"R2=%02X ",R[2]);
    if(debug & 0x00000008) fprintf(fp,"R3=%02X ",R[3]);
	if(debug & 0x00000010) fprintf(fp,"R4=%02X ",R[4]);
	if(debug & 0x00000020) fprintf(fp,"R5=%02X ",R[5]);
	if(debug & 0x00000040) fprintf(fp,"R6=%02X ",R[6]);
	if(debug & 0x00000080) fprintf(fp,"R7=%02X ",R[7]);
    if(debug & 0x00000100) fprintf(fp,"R8=%02X ",R[8]);
    if(debug & 0x00000200) fprintf(fp,"R9=%02X ",R[9]);
    if(debug & 0x00000400) fprintf(fp,"R10=%02X ",R[10]);
    if(debug & 0x00000800) fprintf(fp,"R11=%02X ",R[11]);
	if(debug & 0x00001000) fprintf(fp,"R12=%02X ",R[12]);
	if(debug & 0x00002000) fprintf(fp,"R13=%02X ",R[13]);
	if(debug & 0x00004000) fprintf(fp,"R14=%02X ",R[14]);
	if(debug & 0x00008000) fprintf(fp,"R15=%02X ",R[15]);
    if(debug & 0x00100000) fprintf(fp,"CF=%d ",CF);
    if(debug & 0x00200000) fprintf(fp,"ZF=%d ",ZF);
    if(debug & 0x00800000) fprintf(fp,"PC=%3X (%d) ",PC0,PC0);
    if(debug & 0x01000000) fprintf(fp,"SP=%5d ",SP);
    printf("\n");
}

#define EXIT_STACK_OVER() exit_stack_over(__LINE__)
void exit_stack_over(int n) {
    fprintf(fp,"ERROR: stack overflow(%d) in %X\n%s\n",n,PC0,LDEBUG[PC0]);
    exit(-1);
}

int wz_sim() {
    unsigned int code, PC;
    unsigned long long debug;
    unsigned int A,B,ANS;
    int i, b0,b1,b2,b6, cond,FetchBit;
    time = 0;
    PC  = 0;
    CF = ZF = SP = FetchBit = 0;
    R = calloc(MAX_SRAM,sizeof(unsigned char));
    if(R==NULL) exit_error("Out of Memory");
    memcpy(R+0x400,Sbox,256);
    memcpy(R+0x500,iSbox,256);
    memcpy(R+0x600,exkey,240);

    do {
        unsigned char w0, w1, w2, w3;

        time ++;
        code = PCODE[PC];
        if(lwmode) code = ((code<<16)&0x00ff0000) | ((code>>8)&0xffff) | (code&0xff000000);
        debug = PDEBUG[PC];

        PC0 = PC++;
        w0 = code;
        w1 = code>>8;
        w2 = code>>16;
        w3 = code>>24;

        if(w2==0xff) {
            if(debug) print_register(debug);
            break;
        }

        if(FetchBit==0) {
            if(w0 & 0x80) A = R[w0&0x7f];
            else A = w0;
        } else {
            if(w0 & 0x80) A = R[(B<<8)|A];
            else A = w0;
        }

        if(w1 & 0x80) B = R[w1&0x7f];
        else B = w1;

        b0 = w2&1;
        b1 = (w2>>1)&1;
        b2 = (w2>>2)&1;
        b6 = (w2>>6)&1;

        /* JUMP/CALL cond */
        cond = 0;
        if(b2==0 && b1==0) cond = 1;
        if(b1==1 && CF==b0) cond = 1;
        if(b2==1 && ZF==b0) cond = 1;

        /* RETURN */
        if(w2&0x20 && cond) {
            if(SP==0) EXIT_STACK_OVER();
            PC = STACK[--SP];
        }

        /* CALL */
        if(w2&0x08 && cond) {
            if(SP+1>=MAX_SP) EXIT_STACK_OVER();
            STACK[SP++] = PC;
        }

        /* JUMP */
        if(w2&0x10 && cond) PC = ((B&0x7f)<<7) | (A&0x7f);

        /* ADD etc */
	    if(b1) ANS = A + (~B&0xff);
        else ANS = A + B;
        if(b0) ANS += 1;
        else ANS = b2 ? ANS+CF : ANS;

//printf("PC=%d %08X %s",PC0,code,LDEBUG[PC0]);

        CF = (ANS>>8)&1;

        /* LOGIC etc */
        if(w2&0x40) {
            switch(w2&0x3) {
            case 0: ANS = A; break;
            case 1: ANS = A & B; break;
            case 2: ANS = A ^ B; break;
            case 3: ANS = A | B; break;
            }
        }
 
        if(b6==1 && b2==1 && b1==0 && b0==1) FetchBit = 1; /* FETCH */
        else FetchBit = 0;

        ANS &= 0xff;
        if(ANS==0) ZF = 1;
        else ZF = 0;

        /* OUTPUT */
       if((w2&7)==7) {}

       if(b6==1 && b2==1 && b1==0 && b0==0) { /* STORE */
           if(w3&0x80) R[(w3&0x7f)<<8|B] = ANS;
       } else {
           if(w3&0x80) R[w3&0x7f] = ANS;
       }

       if(debug) print_register(debug);

    } while(1);

    free(R);
    free(PCODE);
    free(PDEBUG);
    for(i=0 ; i<adr_line ; i++) free(LDEBUG[i]);
    free(LDEBUG);

	return 0;
}
