/* WZeta assembler & simulator
   WZeta Project https://wzeta.idletime.tokyo/
   Ver0.0 2019/4/23
   Ver0.1 2019/5/18
   Ver0.2 2019/5/29 
   Ver0.3 2019/5/30
   Ver0.4 2019/6/3
   Ver0.5 2019/6/7
   Naoki Hirayama
 */
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include "wzsim.h"

static char ver[] = "0.5";
extern unsigned long long time;

int lwmode;
unsigned int mode;
FILE *fp;

void exit_error(char* msg) {
    perror(msg);
    exit(-1);
}

int main(int argc,char *argv[]) {
    int exit_code;
    int len;
    char* filename = "";

    fp = stdout;
    mode = 0;
    exit_code = -1;

    if(sizeof(unsigned long long)<8)
        exit_error("unsigned long long is not 8byte");

    len = strlen(argv[0]);
    if(argv[0][len-5]=='l') lwmode = 1;
    else lwmode = 0;
	
	if(argc == 1) {
        printf("wzsim(lwsim) version %s : ",ver);
        if(lwmode==1) printf("    lwsim mode\n\n");
        else printf("    wzsim mode\n\n");
        printf("Usage: wzsim <a,A,s,S> [<filename>]\n");
        printf("    a  -  assemble > filename.mem\n");
        printf("    A  -  assemble > stdout\n");
        printf("    Ad -  assemble with debug info > stdout\n");
        printf("    s  -  simulation > filename.res\n");
        printf("    S  -  simulation > stdout\n");
        printf("wzsim <- wzsim mode , lwsim <- lwsim mode\n");
        return -1;
    }


    if(argc == 2) {
        wz_asm("infile");
        wz_sim();
    }
	if(argv[1][0]=='a' || argv[1][0]=='s') {
        len = strlen(argv[2]);
        filename = (char*)malloc(len+4);
        if(filename==NULL) exit_error("malloc()");
        strcpy(filename,argv[2]);
        if(filename[len-3]!='a' || filename[len-2]!='w' || filename[len-1]!='z') {
            printf("file extention is not asm\n");
            return -1;
        }
        filename[len-4] = '\0';
    }
	if(argc==3 && argv[1][0]=='a') {
        mode |= M_BIN;
        strcat(filename,".mem");
        fp = fopen(filename,"w");
        free(filename);
        if(fp==NULL) exit_error("fopen()");
        wz_asm(argv[2]);
        fclose(fp);
		return 0;
    }
	if(argc==3 && argv[1][0]=='A') {
        mode |= M_STDOUT;
        if(argv[1][1]=='d') mode |= M_DEBUG;
        wz_asm(argv[2]);
		return 0;
    }
	if(argc==3 && argv[1][0]=='s') {
        strcat(filename,".res");
        fp = fopen(filename,"w");
        free(filename);
        if(fp==NULL) exit_error("fopen()");
        wz_asm(argv[2]);
        exit_code = wz_sim();
    }
	if(argc==3 && argv[1][0]=='S') {
        wz_asm(argv[2]);
        exit_code = wz_sim();
    }
    printf("Sim end. end code = %d cyc = %llu\n",exit_code,time);
	if(fp!=stdout) fprintf(fp,"#Sim end. end code = %d cyc = %llu\n",exit_code,time);
	fclose(fp);
    return 0;
}
